<?php
// Functions dealing with URLs and page paths.
    // Safely parses parameters from the URL query string.
    function getQueryStringParams() {
        $eqs = explode('&', $_SERVER['QUERY_STRING']);
        $array = [];
        foreach($eqs as $e) {
          if($e === '') continue;
          $pair = explode('=', $e, 2);
          if($pair[0] === '') continue;
          $array[$pair[0]] = (string) $pair[1];
        }
        return $array;
    }
    
    // Delimiter-respecting concatenation of paths (as in a file path or url).
    function pathConcat($first, ...$paths) {
        $buf = rtrim($first, '/\\');
        foreach($paths as $p)
            $buf .= '/' . trim($p, '/\\');
        return $buf;
    }
    
    // Normalizing concatenation of paths. 
    // Unlike pathConcat, this variant recognizes '.' and '..'
    function normalPathConcat($first, ...$paths) {
        $buf = rtrim($first, '/\\');
        foreach($paths as $p)
            $buf .= '/' . trim($p, '/\\');
        return normalizePath($buf);
    }
    
    // Converts a path to normal form.
    // A path is in normal form if:
    //  - All separators are forward slashes;
    //  - There is no trailing separator;
    //  - There are no empty or '.' components;
    //  - For absolute paths, there are no '..' components. 
    //    For relative paths, there are be zero or more '..' components before the first ordinary component.
    // A path in normal form thus consists of an optional leading (/)?
    // A canonical path is an absolute path in normal form.
    function normalizePath($path) {
        $stack = []; 
        $up = 0;    // Number of times to output ".."
        //$parts = explode('/', $path);
        //foreach($parts as $part) {
        for($part = strtok($path, '/'); $part !== false; $part = strtok('/')) {
            switch($part) {
              case '': 
              case '.': 
                continue;
              case '..': 
                if(array_pop($stack) === null) 
                  $up++;
                break;
              default: 
                $stack[] = $part;
            }
        }
        if($path[0] === '/') $prefix = '/';
        else {
          $prefix = ''; 
          while($up--) $prefix .= '../';
        }
        return $prefix . join('/', $stack);
    }
    
    // Returns the normalized page path. 
    // - Leading/trailing whitespace is trimmed; 
    // - all spaces are replaced with underscores; 
    // - all path components have their first letter capitalized if they start with a letter.
    function getNormalizedPageName($page) {
        return strtr(ucwords(trim($page), '/'), ' ', '_');
    }
    
    // Converts a path in our custom link markup into one suitable for the href 
    // attribute of an HTML <a> element.
    function linkPathToHref($path) {
        global $page_dir;   // The directory of the current page
        switch($path[0]) {
            case '!': return substr($path, 1);                                                         // [[!foo]] -> raw href
            case '/':
              return pathConcat(internal_link_prefix, getNormalizedPageName($path));                   // [[/foo]] -> absolute page
            default:
              //return normalizePath(getNormalizedPageName($path));
              $canonical_page_path = normalPathConcat($page_dir, $path);
              return pathConcat(internal_link_prefix, getNormalizedPageName($canonical_page_path));    // [[foo]] -> relative page
        }
    }
    
    // Converts a path in our custom link markup into a "clean" version suitable as default text.
    function linkPathDefaultText($path) {
        switch($path[0]) {
            case '!':
            case '#': return substr($path, 1);
            default:
                $i = strpos($path, '//');
                if($i !== false)
                    return substr($path, $i+2);
                else if(($i = strrpos($path, '/')) !== false)
                    return substr($path, $i+1);
                else
                    return $path;
        }
    }
    
    // Translates Wikipedia-style free links [[Like|this one]] into HTML <a> elements.
    function pipedLink($piped, $suffix = '', $attrs = []) {
      $pair = explode('|', $piped, 2);
      $pair[0] = trim($pair[0]);
      $href = linkPathToHref($pair[0]);
      $text = isset($pair[1]) ? $pair[1] : linkPathDefaultText($pair[0]);
      return "<a href=\"$href\"" . tagAttrList($attrs) . ">$text$suffix</a>";              // wrapTag inlined for performance
    }
    
// HTML-related helper functions.
    // Generates a list of HTML attributes from the key-value pairs in $attrs.
    function tagAttrList(array $attrs) {
      $eqs = "";
      foreach($attrs as $at=>$vl)
        $eqs .= " $at=\"$vl\"";
      return $eqs;
    }
    
    // Wraps content in an HTML tag with custom attributes.
    function wrapTag($content, $tag, ...$attrs) {
      $html = "<$tag";
      foreach ($attrs as $a)
        $html .= tagAttrList($a);
      $html .= ">$content</$tag>";
      return $html;
    }
    
    function emptyTag($tag, ...$attrs) {
      switch($tag) {
        case 'br':
        case 'link':
        case 'img':
            $attrList = '';
            foreach ($attrs as $a)
                $attrList .= tagAttrList($a);
            return "<$tag$attrList>";
        default:
            return wrapTag('', $tag, ...$attrs);
      }
    }
    
    function error($msg) {
      return wrapTag($msg, 'span', ['class'=>'error']);
    }
?>