<?php
    require_once 'php/link.php';
    
    define("directive_regex", '`{{([@$]?)([A-Za-z0-9_-]+)\s*(.*?)}}(\s*)`'); // Regex used for directives markup.
    /* Directives:
        {{@property value}}
    */
    define("link_regex", '%\[\[(.*?)\]\]([A-Za-z0-9_-]*)%');                // Regex for link markup.
    
    // Holds site-wide content information.
    class Site {
        public $title;          // Site name
        public $slogan;         // Site slogan
        public $copyright;      // Copyright notice
        public $specialPages;   // Associative array containing paths of special pages
        
        public function __construct() {}
        public function addSpecialPage($key, $value) {
            $this->$specialPages[$key] = pathConcat(content_root, $value);
        }
    }

    // Represents the page we're showing.
    class Page {
        public $site;       // The Site object
        public $page;       // The page path
        public $file;       // File containing page content.
        public $properties;
        public $content;
        public $isIncluded; // True if this is being included in another page
        
        public function __construct($site, $name, $included = false) {
            $this->site = $site;
            $this->page = $name;
            $this->isIncluded = $included;
            $this->file = pathConcat(content_root, rawurldecode($name));
        }
        
        public function exists() {return file_exists($this->file);}
        
        // This is where we actually read the file and parse its markup.
        // We need to handle 404 errors here as well.
        public function parseMarkup() {
          $path = $this->file;
          if(!file_exists($path)) {
            $path = $this->site->specialPages['404'];
            http_response_code(404);
          }
          $markup = file_get_contents($path);
          $markup = preg_replace_callback(link_regex, function($match) {return pipedLink($match[1], $match[2]);}, $markup);
          $markup = preg_replace_callback(directive_regex, [$this, 'directive'], $markup);
          if(!isset($this->properties['title']))
            $this->properties['title'] = strtr(basename($path), '_', ' ');
          return $this->content = $markup;
        }
        
        // Returns whether or not the property named $key is an array property.
        static function isArrayProperty($key) {
          switch($key) {
            case 'tag':
            case 'nav-item':
            case 'path':
              return true;
            default:
              return false;
          }
        }
        
        // Callback for handling {{directives}}.
        function directive($match) {
          $trailingSpace = empty($match[4]) ? '' : ' ';
          switch($match[1]) {
            case '@':
                $val = isset($match[3]) ? $match[3] : '';
                $this->setProperty($match[2], $val);
                return '';
            case '$':
                return $this->getPropertyText($match[2]) . $trailingSpace;
          }
          
          switch($match[2]) {
            case 'include':
                global $page_dir; // Holds the current directory in the page hierarchy (used as a prefix for relative internal links)
                
                $inc_path = getNormalizedPageName($match[3]);
                $inc_page = new Page($this->site, $inc_path, true);
                if(!$inc_page->exists()) return error("Included page not found: $inc_path");
                $inc_page->properties = $this->properties;
                
                // kludge: $page_dir is global, so we have to save it locally
                // while we temporarily change it for the included page
                $pushed_dir = $page_dir;
                $page_dir = dirname($inc_page->page);
                $inc_content = $inc_page->parseMarkup() . $trailingSpace;
                $page_dir = $pushed_dir;
                return $inc_content;
            case 'redirect':
                if(!$this->isIncluded) {
                    $redir_url = pathConcat(url_root, internal_link_prefix, $match[3]);
                    header("Location: $redir_url?from={$this->page}", true, 303);
                    die();
                }
            default: 
                return '';
          }
        }
        
        // Sets a page property.
        function setProperty($key, $val) {
            if(self::isArrayProperty($key))
                $this->properties[$key][] = $val;
            else
                $this->properties[$key] = $val;
        }
        
        // Returns the value of a property if it is defined; or returns a 
        // well-typed default for undefined properties.
        function getProperty($key) {
            if(isset($this->properties[$key])) return $this->properties[$key];
            else return self::isArrayProperty($key) ? [] : '';
        }
        
        // Returns a string representing a page property.
        // For array properties, returns the elements of the array se;
        // otherwise does an ordinary string conversion.
        function getPropertyText($key) {
            if(isset($this->properties[$key])) {
              $val = $this->properties[$key];
              if(is_array($val)) {
                return implode(' ', $val);
              }
              else return $val;
            }
            else return '';
        }
    }
    
    // Takes the {nav-item} and {nav-active} properties of a page and outputs HTML for a navigation bar.
    function buildNavbar($page, $attrs) {
        echo "<div $attrs>";
        $items = $page->getProperty('nav-item');
        $active = $page->getProperty('nav-active');
        $i = 0;
        foreach($items as $item) {
            $aclass = ($i == $active) ? "active" : "inactive";
            echo pipedLink($item, '', ['class'=>$aclass]);
            $i++;
        }
        echo "</div>";
    }
    
    // Outputs the 'logical path' of a page, taken from its {path} array property.
    function buildPagePath($page) {
        $path = $page->getProperty('path');
        foreach($path as $element) {
            echo '/';
            echo pipedLink($element);
        }
    }
    
    function buildTags($page) {
        $tags = $page->getProperty('tag');
        if(empty($tags)) return;
        echo '<b>Tags: </b>';
        foreach($tags as $i=>$tag) {
            if($i>0) echo ' | ';
            echo pipedLink($tag);
        }
    }
    
?>